@@ -1,21 +1,21 @@
 /* vim:set ts=2 sw=2 sts=2 et: */
-/* 
+/*
    IGraph library.
-   Copyright (C) 2006-2012  Tamas Nepusz <ntamas@gmail.com>
-   
+   Copyright (C) 2006-2023  Tamas Nepusz <ntamas@gmail.com>
+
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
-   
+
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-   
+
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA 
+   Foundation, Inc.,  51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA
 
 */
@@ -45,33 +45,33 @@
 /**
  * \defgroup python_interface Python module implementation
  * \brief Functions implementing a Python interface to \a igraph
- * 
+ *
  * These functions provide a way to access \a igraph functions from Python.
  * It should be of interest of \a igraph developers only. Classes, functions
  * and methods exposed to Python are still to be documented. Until it is done,
  * just type the following to get help about \a igraph functions in Python
  * (assuming you have \c igraph.so somewhere in your Python library path):
- * 
+ *
  * \verbatim
 import igraph
 help(igraph)
 help(igraph.Graph)
 \endverbatim
- * 
+ *
  * Most of the functions provided here share the same calling conventions
  * (which are determined by the Python/C API). Since the role of the
  * arguments are the same across many functions, I won't explain them
  * everywhere, just give a quick overview of the common argument names here.
- * 
+ *
  * \param self the Python igraph.Graph object the method is working on
  * \param args pointer to the Python tuple containing the arguments
  * \param kwds pointer to the Python hash containing the keyword parameters
  * \param type the type object of a Python igraph.Graph object. Used usually
  * in constructors and class methods.
- * 
+ *
  * Any arguments not documented here should be mentioned at the documentation
  * of the appropriate method.
- * 
+ *
  * The functions which implement a Python method always return a pointer to
  * a \c PyObject. According to Python conventions, this is \c NULL if and
  * only if an exception was thrown by the method (or any of the functions
@@ -80,7 +80,7 @@ help(igraph.Graph)
  * returning a \c NULL value, because this is the same for every such method.
  * The conclusion is that a method can return \c NULL even if I don't state
  * it explicitly.
- * 
+ *
  * Also please take into consideration that I'm documenting the C calls
  * with the abovementioned parameters here, and \em not the Python methods
  * which are presented to the user using the Python interface of \a igraph.
@@ -89,11 +89,11 @@ help(igraph.Graph)
  * or use \c pydoc to generate a formatted version.
  *
  * \section weakrefs The usage of weak references in the Python interface
- * 
+ *
  * Many classes implemented in the Python interface (e.g. VertexSeq, Vertex...)
  * use weak references to keep track of the graph they are referencing to.
  * The use of weak references is twofold:
- * 
+ *
  * -# If we assign a VertexSeq or a Vertex of a given graph to a local
  *    variable and then destroy the graph, real references keep the graph
  *    alive and do not return the memory back to Python.
@@ -117,7 +117,7 @@ help(igraph.Graph)
 /**
  * Whether the module was initialized already
  */
-static igraph_bool_t igraphmodule_initialized = 0;
+static igraph_bool_t igraphmodule_initialized = false;
 
 /**
  * Module-specific global variables
@@ -165,7 +165,7 @@ igraph_error_t igraphmodule_igraph_progress_hook(const char* message, igraph_rea
       }
     }
   }
-  
+
   return IGRAPH_SUCCESS;
 }
 
@@ -182,7 +182,7 @@ igraph_error_t igraphmodule_igraph_status_hook(const char* message, void*data) {
         return IGRAPH_INTERRUPTED;
     }
   }
-  
+
   return IGRAPH_SUCCESS;
 }
 
@@ -238,10 +238,10 @@ PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwd
   igraph_vector_int_t result;
   igraph_matrix_t resmat;
   Py_ssize_t no_of_nodes, i;
-  
+
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!|O", kwlist, &PyList_Type, &vs, &coords))
     return NULL;
-  
+
   no_of_nodes = PyList_Size(vs);
   if (igraph_matrix_init(&mtrx, no_of_nodes, 2)) {
     igraphmodule_handle_igraph_error();
@@ -279,7 +279,7 @@ PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwd
       igraph_matrix_destroy(&mtrx);
       return NULL;
     }
-    
+
     if (!PyNumber_Check(o1) || !PyNumber_Check(o2)) {
       PyErr_SetString(PyExc_TypeError, "vertex coordinates must be numeric");
       Py_DECREF(o2);
@@ -322,7 +322,7 @@ PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwd
       igraph_matrix_destroy(&mtrx);
       igraph_vector_int_destroy(&result);
       return NULL;
-    }    
+    }
     o = igraphmodule_vector_int_t_to_PyList(&result);
     igraph_vector_int_destroy(&result);
   } else {
@@ -336,11 +336,11 @@ PyObject* igraphmodule_convex_hull(PyObject* self, PyObject* args, PyObject* kwd
       igraph_matrix_destroy(&mtrx);
       igraph_matrix_destroy(&resmat);
       return NULL;
-    }        
+    }
     o = igraphmodule_matrix_t_to_PyList(&resmat, IGRAPHMODULE_TYPE_FLOAT);
     igraph_matrix_destroy(&resmat);
   }
-  
+
   igraph_matrix_destroy(&mtrx);
 
   return o;
@@ -358,7 +358,9 @@ PyObject* igraphmodule_community_to_membership(PyObject *self,
   if (!PyArg_ParseTupleAndKeywords(args, kwds, "O!nn|O", kwlist,
       &PyList_Type, &merges_o, &nodes, &steps, &return_csize)) return NULL;
 
-  if (igraphmodule_PyList_to_matrix_int_t_with_minimum_column_count(merges_o, &merges, 2)) return NULL;
+  if (igraphmodule_PyList_to_matrix_int_t_with_minimum_column_count(merges_o, &merges, 2, "merges")) {
+    return NULL;
+  }
 
   CHECK_SSIZE_T_RANGE(nodes, "number of nodes");
   CHECK_SSIZE_T_RANGE(steps, "number of steps");
@@ -518,19 +520,47 @@ PyObject* igraphmodule_is_graphical_degree_sequence(PyObject *self,
 }
 
 
+static PyObject* igraphmodule_i_is_graphical_or_bigraphical(
+  PyObject *self, PyObject *args, PyObject *kwds, igraph_bool_t is_bigraphical
+);
+
 PyObject* igraphmodule_is_graphical(PyObject *self, PyObject *args, PyObject *kwds) {
-  static char* kwlist[] = { "out_deg", "in_deg", "loops", "multiple", NULL };
+  return igraphmodule_i_is_graphical_or_bigraphical(self, args, kwds, /* is_bigraphical = */ false);
+}
+
+PyObject* igraphmodule_is_bigraphical(PyObject *self, PyObject *args, PyObject *kwds) {
+  return igraphmodule_i_is_graphical_or_bigraphical(self, args, kwds, /* is_bigraphical = */ true);
+}
+
+static PyObject* igraphmodule_i_is_graphical_or_bigraphical(
+  PyObject *self, PyObject *args, PyObject *kwds, igraph_bool_t is_bigraphical
+) {
+  static char* kwlist_graphical[] = { "out_deg", "in_deg", "loops", "multiple", NULL };
+  static char* kwlist_bigraphical[] = { "degrees1", "degrees2", "multiple", NULL };
   PyObject *out_deg_o = 0, *in_deg_o = 0;
   PyObject *loops = Py_False, *multiple = Py_False;
   igraph_vector_int_t out_deg, in_deg;
   igraph_bool_t is_directed, result;
   int allowed_edge_types;
+  igraph_error_t retval;
 
-  if (!PyArg_ParseTupleAndKeywords(args, kwds, "O|OOO", kwlist,
-      &out_deg_o, &in_deg_o, &loops, &multiple))
-    return NULL;
+  if (is_bigraphical) {
+    if (!PyArg_ParseTupleAndKeywords(
+        args, kwds, "OO|O", kwlist_bigraphical,
+        &out_deg_o, &in_deg_o, &multiple
+    )) {
+      return NULL;
+    }
+  } else {
+    if (!PyArg_ParseTupleAndKeywords(
+        args, kwds, "O|OOO", kwlist_graphical,
+        &out_deg_o, &in_deg_o, &loops, &multiple
+    )) {
+      return NULL;
+    }
+  }
 
-  is_directed = (in_deg_o != 0 && in_deg_o != Py_None);
+  is_directed = (in_deg_o != 0 && in_deg_o != Py_None) || is_bigraphical;
 
   if (igraphmodule_PyObject_to_vector_int_t(out_deg_o, &out_deg))
     return NULL;
@@ -548,7 +578,11 @@ PyObject* igraphmodule_is_graphical(PyObject *self, PyObject *args, PyObject *kw
     allowed_edge_types |= IGRAPH_MULTI_SW;
   }
 
-  if (igraph_is_graphical(&out_deg, is_directed ? &in_deg : 0, allowed_edge_types, &result)) {
+  retval = is_bigraphical
+    ? igraph_is_bigraphical(&out_deg, is_directed ? &in_deg : 0, allowed_edge_types, &result)
+    : igraph_is_graphical(&out_deg, is_directed ? &in_deg : 0, allowed_edge_types, &result);
+
+  if (retval) {
     igraphmodule_handle_igraph_error();
     igraph_vector_int_destroy(&out_deg);
     if (is_directed) {
@@ -637,6 +671,64 @@ PyObject* igraphmodule_split_join_distance(PyObject *self,
   return Py_BuildValue("nn", (Py_ssize_t)distance12, (Py_ssize_t)distance21);
 }
 
+
+/** \ingroup python_interface_graph
+ * \brief Compute weights for Uniform Manifold Approximation and Projection (UMAP)
+ * \return the weights given that graph
+ * \sa igraph_umap_compute_weights
+ */
+PyObject *igraphmodule_umap_compute_weights(
+    PyObject *self, PyObject * args, PyObject * kwds)
+{
+  static char *kwlist[] = { "graph", "dist", NULL };
+  igraph_vector_t *dist = 0;
+  igraph_vector_t weights;
+  PyObject *dist_o = Py_None, *graph_o = Py_None;
+  PyObject *result_o;
+  igraphmodule_GraphObject * graph;
+
+  if (!PyArg_ParseTupleAndKeywords(args, kwds, "OO", kwlist, &graph_o, &dist_o))
+    return NULL;
+
+  /* Initialize distances */
+  if (dist_o != Py_None) {
+    dist = (igraph_vector_t*)malloc(sizeof(igraph_vector_t));
+    if (!dist) {
+      PyErr_NoMemory();
+      return NULL;
+    }
+    if (igraphmodule_PyObject_to_vector_t(dist_o, dist, 0)) {
+      free(dist);
+      return NULL;
+    }
+  }
+
+  /* Extract graph from Python object */
+  graph = (igraphmodule_GraphObject*)graph_o;
+
+  /* Initialize weights */
+  if (igraph_vector_init(&weights, 0)) {
+      igraph_vector_destroy(dist); free(dist);
+      PyErr_NoMemory();
+      return NULL;
+  }
+
+  /* Call the function */
+  if (igraph_layout_umap_compute_weights(&graph->g, dist, &weights)) {
+      igraph_vector_destroy(&weights);
+      igraph_vector_destroy(dist); free(dist);
+      PyErr_NoMemory();
+      return NULL;
+  }
+  igraph_vector_destroy(dist); free(dist);
+
+  /* Convert output to Python list */
+  result_o = igraphmodule_vector_t_to_PyList(&weights, IGRAPHMODULE_TYPE_FLOAT);
+  igraph_vector_destroy(&weights);
+  return (PyObject *) result_o;
+}
+
+
 #define LOCALE_CAPSULE_TYPE "igraph._igraph.locale_capsule"
 
 void igraphmodule__destroy_locale_capsule(PyObject *capsule) {
@@ -646,7 +738,7 @@ void igraphmodule__destroy_locale_capsule(PyObject *capsule) {
   }
 }
 
-PyObject* igraphmodule__enter_safelocale(PyObject* self) {
+PyObject* igraphmodule__enter_safelocale(PyObject* self, PyObject* Py_UNUSED(_null)) {
   igraph_safelocale_t* loc;
   PyObject* capsule;
 
@@ -688,7 +780,7 @@ PyObject* igraphmodule__exit_safelocale(PyObject *self, PyObject *capsule) {
 /** \ingroup python_interface
  * \brief Method table for the igraph Python module
  */
-static PyMethodDef igraphmodule_methods[] = 
+static PyMethodDef igraphmodule_methods[] =
 {
   {"community_to_membership", (PyCFunction)igraphmodule_community_to_membership,
     METH_VARARGS | METH_KEYWORDS,
@@ -734,6 +826,19 @@ static PyMethodDef igraphmodule_methods[] =
     "@return: C{True} if there exists some graph that can realize the given degree\n"
     "  sequence, C{False} otherwise.\n"
   },
+  {"is_bigraphical", (PyCFunction)igraphmodule_is_bigraphical,
+    METH_VARARGS | METH_KEYWORDS,
+    "is_bigraphical(degrees1, degrees2, multiple=False)\n--\n\n"
+    "Returns whether two sequences of integers can be the degree sequences of a\n"
+    "bipartite graph.\n\n"
+    "The bipartite graph may or may not have multiple edges, depending\n"
+    "on the allowed edge types in the remaining arguments.\n\n"
+    "@param degrees1: the list of degrees in the first partition.\n"
+    "@param degrees2: the list of degrees in the second partition.\n"
+    "@param multiple: whether multiple edges are allowed.\n"
+    "@return: C{True} if there exists some bipartite graph that can realize the\n"
+    "  given degree sequences with or without multiple edges, C{False} otherwise.\n"
+  },
   {"is_graphical", (PyCFunction)igraphmodule_is_graphical,
     METH_VARARGS | METH_KEYWORDS,
     "is_graphical(out_deg, in_deg=None, loops=False, multiple=False)\n--\n\n"
@@ -764,6 +869,35 @@ static PyMethodDef igraphmodule_methods[] =
     "@return: C{True} if there exists some simple graph that can realize the given\n"
     "  degree sequence, C{False} otherwise.\n"
   },
+  {"umap_compute_weights", (PyCFunction)igraphmodule_umap_compute_weights,
+   METH_VARARGS | METH_KEYWORDS,
+   "umap_compute_weights(graph, dist)\n--\n\n"
+   "Compute undirected UMAP weights from directed distance graph.\n"
+   "UMAP is a layout algorithm that usually takes as input a directed\n"
+   "distance graph, for instance a k nearest neighbor graph based on Euclidean\n"
+   "distance between points in a vector space. The graph is directed\n"
+   "because vertex v1 might consider vertex v2 a close neighbor, but v2\n"
+   "itself might have many neighbors that are closer than v1.\n"
+   "This function computes the symmetrized weights from the distance graph\n"
+   "using union as the symmetry operator. In simple terms, if either vertex\n"
+   "considers the other a close neighbor, they will be treated as close\n"
+   "neighbors.\n\n"
+   "This function can be used as a separate preprocessing step to\n"
+   "Graph.layout_umap(). For efficiency reasons, the returned weights have the\n"
+   "same length as the input distances, however because of the symmetryzation\n"
+   "some information is lost. Therefore, the weight of one of the edges is set\n"
+   "to zero whenever edges in opposite directions are found in the input\n"
+   "distance graph. You can pipe the output of this function directly into\n"
+   "Graph.layout_umap() as follows:\n"
+   "C{weights = igraph.umap_compute_weights(graph, dist)}\n"
+   "C{layout = graph.layout_umap(weights=weights)}\n\n"
+   "@param graph: directed graph to compute weights for.\n"
+   "@param dist: distances associated with the graph edges.\n"
+   "@return: Symmetrized weights associated with each edge. If the distance\n"
+   "  graph has both directed edges between a pair of vertices, one of the\n"
+   "  returned weights will be set to zero.\n\n"
+   "@see: Graph.layout_umap()\n"
+  },
   {"set_progress_handler", igraphmodule_set_progress_handler, METH_O,
       "set_progress_handler(handler)\n--\n\n"
       "Sets the handler to be called when igraph is performing a long operation.\n"
@@ -949,7 +1083,7 @@ PyObject* PyInit__igraph(void)
   PyModule_AddObject(m, "EdgeSeq", (PyObject*)igraphmodule_EdgeSeqType);
   PyModule_AddObject(m, "Vertex", (PyObject*)igraphmodule_VertexType);
   PyModule_AddObject(m, "VertexSeq", (PyObject*)igraphmodule_VertexSeqType);
- 
+
   /* Internal error exception type */
   igraphmodule_InternalError =
     PyErr_NewException("igraph._igraph.InternalError", PyExc_Exception, NULL);
@@ -1024,7 +1158,7 @@ PyObject* PyInit__igraph(void)
   igraph_set_status_handler(igraphmodule_igraph_status_hook);
   igraph_set_warning_handler(igraphmodule_igraph_warning_hook);
   igraph_set_interruption_handler(igraphmodule_igraph_interrupt_hook);
-  
+
   /* initialize attribute handlers */
   igraphmodule_initialize_attribute_handler();
 
