@@ -8,81 +8,9 @@ This module provides the framework for altering and querying igraph parameters
 as well as saving them to and retrieving them from disk.
 """
 
-import sys
-
-if sys.version_info < (3, 2):
-    from configparser import SafeConfigParser as ConfigParser
-else:
-    from configparser import ConfigParser
-
-import platform
 import os.path
 
-
-def get_platform_image_viewer():
-    """Returns the path of an image viewer on the given platform.
-
-    Deprecated since igraph 0.9.1 and will be removed in 0.10.0.
-
-    @deprecated: This function was only used by the now-deprecated C{show()}
-        method of the Plot class.
-    """
-    plat = platform.system()
-    if plat == "Darwin":
-        # Most likely Mac OS X
-        return "open"
-    elif plat == "Linux":
-        # Linux has a whole lot of choices, try to find one
-        choices = [
-            "eog",
-            "gthumb",
-            "gqview",
-            "kuickshow",
-            "xnview",
-            "display",
-            "gpicview",
-            "gwenview",
-            "qiv",
-            "gimv",
-            "ristretto",
-            "geeqie",
-            "eom",
-        ]
-        paths = ["/usr/bin", "/bin"]
-        for path in paths:
-            for choice in choices:
-                full_path = os.path.join(path, choice)
-                if os.path.isfile(full_path):
-                    return full_path
-        return ""
-    elif plat == "FreeBSD":
-        # FreeBSD also has a whole lot of choices, try to find one
-        choices = [
-            "eog",
-            "gthumb",
-            "geeqie",
-            "display",
-            "gpicview",
-            "gwenview",
-            "qiv",
-            "gimv",
-            "ristretto",
-            "geeqie",
-            "eom",
-        ]
-        paths = ["%%LOCALBASE%%/bin"]
-        for path in paths:
-            for choice in choices:
-                full_path = os.path.join(path, choice)
-                if os.path.isfile(full_path):
-                    return full_path
-        return ""
-    elif plat == "Windows" or plat == "Microsoft":  # Thanks to Dale Hunscher
-        # Use the built-in Windows image viewer, if available
-        return "start"
-    else:
-        # Unknown system
-        return ""
+from configparser import ConfigParser
 
 
 class Configuration:
@@ -148,25 +76,15 @@ class Configuration:
         - B{verbose}: whether L{igraph} should talk more than really necessary.
           For instance, if set to C{True}, some functions display progress bars.
 
-    Application settings
-    --------------------
-
-    These settings specify the external applications that are possibly
-    used by C{igraph}. They are all stored in section C{apps}.
-
-        - B{image_viewer}: image viewer application. If set to an empty string,
-          it will be determined automatically from the platform C{igraph} runs
-          on. On Mac OS X, it defaults to the Preview application. On Linux,
-          it chooses a viewer from several well-known Linux viewers like
-          C{gthumb}, C{kuickview} and so on (see the source code for the full
-          list). On Windows, it defaults to the system's built-in image viewer.
-
     Plotting settings
     -----------------
 
     These settings specify the default values used by plotting functions.
     They are all stored in section C{plotting}.
 
+        - B{backend}: either "cairo" if you want to use Cairo for plotting
+          or "matplotlib" if you want to use the Matplotlib plotting backend.
+
         - B{layout}: default graph layout algorithm to be used.
 
         - B{mark_groups}: whether to mark the clusters by polygons when
@@ -187,7 +105,8 @@ class Configuration:
     embedded (e.g., IPython and its Qt console). These settings are stored in
     section C{shell}.
 
-        - B{ipython.inlining.Plot}: whether to show instances of the L{Plot} class
+        - B{ipython.inlining.Plot}: whether to show instances of the
+          L{Plot<drawing.Plot>} class
           inline in IPython's console if the console supports it. Default: C{True}
     """
 
@@ -254,7 +173,7 @@ class Configuration:
     _definitions = {
         "general.shells": {"default": "IPythonShell,ClassicPythonShell"},
         "general.verbose": {"default": True, "type": "boolean"},
-        "apps.image_viewer": {"default": get_platform_image_viewer()},
+        "plotting.backend": {"default": "cairo"},
         "plotting.layout": {"default": "auto"},
         "plotting.mark_groups": {"default": False, "type": "boolean"},
         "plotting.palette": {"default": "gray"},
@@ -341,7 +260,7 @@ class Configuration:
             # Special case: retrieving all the keys within a section and
             # returning it in a dict
             keys = self._config.items(section)
-            return dict((key, self._get(section, key)) for key, _ in keys)
+            return {key: self._get(section, key) for key, _ in keys}
         else:
             return self._get(section, key)
 
@@ -394,7 +313,7 @@ class Configuration:
         if isinstance(stream, str):
             stream = open(stream, "r")
             file_was_open = True
-        self._config.readfp(stream)
+        self._config.read_file(stream)
         self._filename = getattr(stream, "name", None)
         if file_was_open:
             stream.close()
