@@ -56,6 +56,19 @@ class SimplePropertiesTests(unittest.TestCase):
         )
         self.assertEqual(Graph().eccentricity(), [])
 
+    def testWeightedEccentricity(self):
+        self.assertEqual(
+            self.gfull.eccentricity(weights=[2] * self.gfull.ecount()),
+            [2] * self.gfull.vcount(),
+        )
+        self.assertEqual(
+            self.gempty.eccentricity(weights=[]), [0] * self.gempty.vcount()
+        )
+        self.assertEqual(
+            self.g.eccentricity(weights=range(1, self.g.ecount() + 1)), [4, 5, 6, 6]
+        )
+        self.assertEqual(Graph().eccentricity(), [])
+
     def testRadius(self):
         self.assertEqual(self.gfull.radius(), 1)
         self.assertEqual(self.gempty.radius(), 0)
@@ -64,6 +77,12 @@ class SimplePropertiesTests(unittest.TestCase):
         self.assertEqual(self.tree.radius(), 3)
         self.assertTrue(isnan(Graph().radius()))
 
+    def testWeightedRadius(self):
+        self.assertEqual(self.gfull.radius(weights=[2] * self.gfull.ecount()), 2)
+        self.assertEqual(self.gempty.radius(weights=[]), 0)
+        self.assertEqual(self.g.radius(weights=range(1, self.g.ecount() + 1)), 4)
+        self.assertTrue(isnan(Graph().radius()))
+
     def testTransitivity(self):
         self.assertTrue(self.gfull.transitivity_undirected() == 1.0)
         self.assertTrue(self.tree.transitivity_undirected() == 0.0)
@@ -125,10 +144,6 @@ class DegreeTests(unittest.TestCase):
         self.assertTrue(knn == [9.0] * 10)
         self.assertAlmostEqual(knnk[8], 9.0, places=6)
 
-        # knn works for simple graphs only -- self.g is not simple
-        self.assertRaises(InternalError, self.g.knn)
-
-        # Okay, simplify it and then go on
         g = self.g.copy()
         g.simplify()
 
@@ -139,6 +154,20 @@ class DegreeTests(unittest.TestCase):
         self.assertAlmostEqual(knnk[1], 3, places=6)
         self.assertAlmostEqual(knnk[2], 7 / 3.0, places=6)
 
+    def testKnnNonSimple(self):
+        knn, knnk = self.gfull.knn()
+        self.assertTrue(knn == [9.0] * 10)
+        self.assertAlmostEqual(knnk[8], 9.0, places=6)
+
+        # knn works for non-simple graphs as well
+        knn, knnk = self.g.knn()
+        diff = max(abs(a - b) for a, b in zip(knn, [17 / 5.0, 3, 4, 4]))
+        self.assertAlmostEqual(diff, 0.0, places=6)
+        self.assertEqual(len(knnk), 5)
+        self.assertAlmostEqual(knnk[1], 4, places=6)
+        self.assertAlmostEqual(knnk[2], 3, places=6)
+        self.assertAlmostEqual(knnk[4], 3.4, places=6)
+
     def testDegree(self):
         self.assertTrue(self.gfull.degree() == [9] * 10)
         self.assertTrue(self.gempty.degree() == [0] * 10)
@@ -237,28 +266,67 @@ class CentralityTests(unittest.TestCase):
     def testBetweennessCentrality(self):
         g = Graph.Star(5)
         self.assertTrue(g.betweenness() == [6.0, 0.0, 0.0, 0.0, 0.0])
+
         g = Graph(5, [(0, 1), (0, 2), (0, 3), (1, 4)])
         self.assertTrue(g.betweenness() == [5.0, 3.0, 0.0, 0.0, 0.0])
         self.assertTrue(g.betweenness(cutoff=2) == [3.0, 1.0, 0.0, 0.0, 0.0])
         self.assertTrue(g.betweenness(cutoff=1) == [0.0, 0.0, 0.0, 0.0, 0.0])
+
         g = Graph.Lattice([3, 3], circular=False)
         self.assertTrue(
             g.betweenness(cutoff=2) == [0.5, 2.0, 0.5, 2.0, 4.0, 2.0, 0.5, 2.0, 0.5]
         )
 
+        observed = g.betweenness(sources=[0, 8], targets=[0, 8])
+        self.assertEqual(len(observed), g.vcount())
+        for x, y in zip(
+            observed, [0, 1 / 2, 1 / 6, 1 / 2, 2 / 3, 1 / 2, 1 / 6, 1 / 2, 0]
+        ):
+            self.assertAlmostEqual(x, y)
+        self.assertRaises(
+            ValueError, g.betweenness, cutoff=2, sources=[0, 8], targets=[0, 8]
+        )
+
     def testEdgeBetweennessCentrality(self):
         g = Graph.Star(5)
         self.assertTrue(g.edge_betweenness() == [4.0, 4.0, 4.0, 4.0])
+
         g = Graph(5, [(0, 1), (0, 2), (0, 3), (1, 4)])
         self.assertTrue(g.edge_betweenness() == [6.0, 4.0, 4.0, 4.0])
         self.assertTrue(g.edge_betweenness(cutoff=2) == [4.0, 3.0, 3.0, 2.0])
         self.assertTrue(g.edge_betweenness(cutoff=1) == [1.0, 1.0, 1.0, 1.0])
+
         g = Graph.Ring(5)
         self.assertTrue(g.edge_betweenness() == [3.0, 3.0, 3.0, 3.0, 3.0])
         self.assertTrue(
             g.edge_betweenness(weights=[4, 1, 1, 1, 1]) == [0.5, 3.5, 5.5, 5.5, 3.5]
         )
 
+        g = Graph.Lattice([3, 3], circular=False)
+        observed = g.edge_betweenness(sources=[0, 8], targets=[0, 8])
+        self.assertEqual(len(observed), g.ecount())
+        for x, y in zip(
+            observed,
+            [
+                1 / 2,
+                1 / 2,
+                1 / 6,
+                1 / 3,
+                1 / 6,
+                1 / 3,
+                1 / 6,
+                1 / 3,
+                1 / 3,
+                1 / 2,
+                1 / 6,
+                1 / 2,
+            ],
+        ):
+            self.assertAlmostEqual(x, y)
+        self.assertRaises(
+            ValueError, g.edge_betweenness, cutoff=2, sources=[0, 8], targets=[0, 8]
+        )
+
     def testClosenessCentrality(self):
         g = Graph.Star(5)
         cl = g.closeness()
@@ -564,45 +632,6 @@ class MiscTests(unittest.TestCase):
         g += cc
         self.assertListEqual([], g.chordal_completion())
 
-    def testChordalCompletion(self):
-        g = Graph()
-        self.assertListEqual([], g.chordal_completion())
-
-        g = Graph.Full(3)
-        self.assertListEqual([], g.chordal_completion())
-
-        g = Graph.Full(5)
-        self.assertListEqual([], g.chordal_completion())
-
-        g = Graph.Ring(4)
-        cc = g.chordal_completion()
-        self.assertEqual(len(cc), 1)
-        g += cc
-        self.assertTrue(g.is_chordal())
-        self.assertListEqual([], g.chordal_completion())
-
-        g = Graph.Ring(5)
-        cc = g.chordal_completion()
-        self.assertEqual(len(cc), 2)
-        g += cc
-        self.assertListEqual([], g.chordal_completion())
-
-    def testChordalCompletionWithHints(self):
-        g = Graph.Ring(4)
-        alpha, _ = g.maximum_cardinality_search()
-        cc = g.chordal_completion(alpha=alpha)
-        self.assertEqual(len(cc), 1)
-        g += cc
-        self.assertTrue(g.is_chordal())
-        self.assertListEqual([], g.chordal_completion())
-
-        g = Graph.Ring(5)
-        _, alpham1 = g.maximum_cardinality_search()
-        cc = g.chordal_completion(alpham1=alpham1)
-        self.assertEqual(len(cc), 2)
-        g += cc
-        self.assertListEqual([], g.chordal_completion())
-
     def testConstraint(self):
         g = Graph(4, [(0, 1), (0, 2), (1, 2), (0, 3), (1, 3)])
         self.assertTrue(isinstance(g.constraint(), list))  # TODO check more
@@ -625,7 +654,9 @@ class MiscTests(unittest.TestCase):
         self.assertTrue(g.is_tree())
 
         g = Graph(1, directed=True)
-        self.assertTrue(g.is_tree() and g.is_tree("out") and g.is_tree("in") and g.is_tree("all"))
+        self.assertTrue(
+            g.is_tree() and g.is_tree("out") and g.is_tree("in") and g.is_tree("all")
+        )
 
         g = Graph(5, [(0, 1), (1, 2), (1, 3), (3, 4)])
         self.assertTrue(g.is_tree())
@@ -650,8 +681,7 @@ class MiscTests(unittest.TestCase):
 
         g = Graph.Ring(10)
         self.assertFalse(
-            g.is_tree() or g.is_tree("in") or g.is_tree("out") or
-            g.is_tree("all")
+            g.is_tree() or g.is_tree("in") or g.is_tree("out") or g.is_tree("all")
         )
 
     def testIsChordal(self):
@@ -769,6 +799,40 @@ class PathTests(unittest.TestCase):
             g.distances(weights="weight", target=[2, 3])
             == [row[2:4] for row in expected]
         )
+        self.assertTrue(
+            g.distances(weights="weight", target=[2, 3], algorithm="bellman_ford")
+            == [row[2:4] for row in expected]
+        )
+        self.assertTrue(
+            g.distances(weights="weight", target=[2, 3], algorithm="johnson")
+            == [row[2:4] for row in expected]
+        )
+        self.assertRaises(
+            ValueError,
+            g.distances,
+            weights="weight",
+            target=[2, 3],
+            algorithm="johnson",
+            mode="in",
+        )
+
+    def testGetShortestPath(self):
+        g = Graph(4, [(0, 1), (0, 2), (1, 3), (3, 2), (2, 1)], directed=True)
+        self.assertEqual([0, 1, 3], g.get_shortest_path(0, 3))
+        self.assertEqual([0, 1, 3], g.get_shortest_path(0, 3, output="vpath"))
+        self.assertEqual([0, 2], g.get_shortest_path(0, 3, output="epath"))
+        self.assertRaises(ValueError, g.get_shortest_path, 0, 3, output="x")
+        self.assertRaises(TypeError, g.get_shortest_path, 0)
+
+    def testGetShortestPathManualAlgorithmSelection(self):
+        g = Graph(4, [(0, 1), (0, 2), (1, 3), (3, 2), (2, 1)], directed=True)
+        g.es["weight"] = [1] * g.ecount()
+
+        self.assertEqual([0, 1, 3], g.get_shortest_path(0, 3, algorithm="bellman_ford"))
+        self.assertRaises(ValueError, g.get_shortest_path, 0, 3, algorithm="johnson")
+        self.assertRaises(
+            ValueError, g.get_shortest_path, 0, 3, algorithm="johnson", mode="in"
+        )
 
     def testGetShortestPaths(self):
         g = Graph(4, [(0, 1), (0, 2), (1, 3), (3, 2), (2, 1)], directed=True)
@@ -843,21 +907,21 @@ class PathTests(unittest.TestCase):
 
     def testGetKShortestPaths(self):
         g = Graph(4, [(0, 1), (1, 2), (1, 3), (2, 4), (3, 4), (4, 5)], directed=True)
-    
+
         sps = sorted(g.get_k_shortest_paths(0, 0))
         expected = [[0]]
         self.assertEqual(expected, sps)
-    
+
         sps = sorted(g.get_k_shortest_paths(0, 5, 2))
         expected = [[0, 1, 2, 4, 5], [0, 1, 3, 4, 5]]
         self.assertEqual(expected, sps)
-    
+
         sps = sorted(g.get_k_shortest_paths(1, 4, 2))
         expected = [[1, 2, 4], [1, 3, 4]]
         self.assertEqual(expected, sps)
-    
+
         g = Graph.Lattice([5, 5], circular=False)
-    
+
         sps = sorted(g.get_k_shortest_paths(0, 12, 6))
         expected = [
             [0, 1, 2, 7, 12],
@@ -868,7 +932,7 @@ class PathTests(unittest.TestCase):
             [0, 5, 10, 11, 12],
         ]
         self.assertEqual(expected, sps)
-    
+
         g = Graph.Lattice([100, 100], circular=False)
         sps = sorted(g.get_k_shortest_paths(0, 202, 6))
         expected = [
@@ -880,12 +944,12 @@ class PathTests(unittest.TestCase):
             [0, 100, 200, 201, 202],
         ]
         self.assertEqual(expected, sps)
-    
+
         g = Graph([(0, 1), (1, 2), (0, 2)])
         g.es["weight"] = [0.5, 0.5, 1]
         sps = sorted(g.get_k_shortest_paths(0, 2, 2, weights="weight"))
         self.assertEqual(sorted([[0, 2], [0, 1, 2]]), sorted(sps))
-    
+
     def testGetAllSimplePaths(self):
         g = Graph.Ring(20)
         sps = sorted(g.get_all_simple_paths(0, 10))
@@ -940,6 +1004,21 @@ class PathTests(unittest.TestCase):
         h = g.path_length_hist(False)
         self.assertTrue(h.unconnected == 20)
 
+    def testGetShortestPathsAStar(self):
+        n = 4
+        g = Graph.Lattice((n, n), circular=False)
+        xs, ys = list(range(n)) * n, sum(([i] * n for i in range(n)), [])
+        g.vs["coord"] = list(zip(xs, ys))
+
+        def heuristics(graph, u, v):
+            ux, uy = graph.vs[u]["coord"]
+            vx, vy = graph.vs[v]["coord"]
+            return ((ux - vx) ** 2 + (uy - vy) ** 2) ** 0.5
+
+        self.assertEqual(
+            [0, 1, 5, 6, 10, 11], g.get_shortest_path_astar(0, 11, heuristics)
+        )
+
 
 class DominatorTests(unittest.TestCase):
     def compareDomTrees(self, alist, blist):
@@ -948,7 +1027,7 @@ class DominatorTests(unittest.TestCase):
         """
         if len(alist) != len(blist):
             return False
-        for i, (a, b) in enumerate(zip(alist, blist)):
+        for _i, (a, b) in enumerate(zip(alist, blist)):
             if math.isnan(a) and math.isnan(b):
                 continue
             elif a == b:
@@ -1079,12 +1158,20 @@ class DominatorTests(unittest.TestCase):
 
 
 def suite():
-    simple_suite = unittest.defaultTestLoader.loadTestsFromTestCase(SimplePropertiesTests)
+    simple_suite = unittest.defaultTestLoader.loadTestsFromTestCase(
+        SimplePropertiesTests
+    )
     degree_suite = unittest.defaultTestLoader.loadTestsFromTestCase(DegreeTests)
-    local_transitivity_suite = unittest.defaultTestLoader.loadTestsFromTestCase(LocalTransitivityTests)
-    biconnected_suite = unittest.defaultTestLoader.loadTestsFromTestCase(BiconnectedComponentTests)
+    local_transitivity_suite = unittest.defaultTestLoader.loadTestsFromTestCase(
+        LocalTransitivityTests
+    )
+    biconnected_suite = unittest.defaultTestLoader.loadTestsFromTestCase(
+        BiconnectedComponentTests
+    )
     centrality_suite = unittest.defaultTestLoader.loadTestsFromTestCase(CentralityTests)
-    neighborhood_suite = unittest.defaultTestLoader.loadTestsFromTestCase(NeighborhoodTests)
+    neighborhood_suite = unittest.defaultTestLoader.loadTestsFromTestCase(
+        NeighborhoodTests
+    )
     path_suite = unittest.defaultTestLoader.loadTestsFromTestCase(PathTests)
     misc_suite = unittest.defaultTestLoader.loadTestsFromTestCase(MiscTests)
     dominator_suite = unittest.defaultTestLoader.loadTestsFromTestCase(DominatorTests)
